/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.builders.block;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import buildcraft.api.properties.BuildCraftProperties;

import buildcraft.lib.block.BlockBCBase_Neptune;
import buildcraft.lib.misc.RotationUtil;

public class BlockFrame extends BlockBCBase_Neptune {
    public static final Map<EnumFacing, IProperty<Boolean>> CONNECTED_MAP = BuildCraftProperties.CONNECTED_MAP;

    public static final AxisAlignedBB BASE_AABB = new AxisAlignedBB(4 / 16D, 4 / 16D, 4 / 16D, 12 / 16D, 12 / 16D, 12 / 16D);
    public static final AxisAlignedBB CONNECTION_AABB = new AxisAlignedBB(4 / 16D, 0 / 16D, 4 / 16D, 12 / 16D, 4 / 16D, 12 / 16D);

    public BlockFrame(Material material, String id) {
        super(material, id);
    }

    @Override
    protected void addProperties(List<IProperty<?>> properties) {
        super.addProperties(properties);
        properties.addAll(CONNECTED_MAP.values());
    }

    @Override
    public IBlockState func_176221_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        for (EnumFacing side : CONNECTED_MAP.keySet()) {
            Block block = world.func_180495_p(pos.func_177972_a(side)).func_177230_c();
            state = state.func_177226_a(CONNECTED_MAP.get(side), block instanceof BlockFrame || block instanceof BlockQuarry);
        }
        return state;
    }

    @Override
    public BlockRenderLayer func_180664_k() {
        return BlockRenderLayer.CUTOUT;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_176225_a(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing side) {
        IBlockState actualState = state.func_185899_b(world, pos);
        EnumFacing[] facings = CONNECTED_MAP.keySet().stream()
                .filter(facing -> actualState.func_177229_b(CONNECTED_MAP.get(facing)))
                .toArray(EnumFacing[]::new);
        if (facings.length == 1) {
            return side != facings[0];
        } else if (facings.length == 2 && facings[0] == facings[1].func_176734_d()) {
            return side != facings[0] && side != facings[1];
        }
        return true;
    }

    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        IBlockState actualState = state.func_185899_b(world, pos);
        AtomicReference<AxisAlignedBB> box = new AtomicReference<>(BASE_AABB);
        CONNECTED_MAP.forEach((side, property) -> {
            if (actualState.func_177229_b(property)) {
                box.set(box.get().func_111270_a(RotationUtil.rotateAABB(CONNECTION_AABB, side)));
            }
        });
        return box.get();
    }

    @Override
    public void func_185477_a(IBlockState state, World world, BlockPos pos, AxisAlignedBB entityBox, List<AxisAlignedBB> collidingBoxes, @Nullable Entity entity, boolean isPistonMoving) {
        IBlockState actualState = state.func_185899_b(world, pos);
        func_185492_a(pos, entityBox, collidingBoxes, BASE_AABB);
        CONNECTED_MAP.keySet().stream()
                .filter(side -> actualState.func_177229_b(CONNECTED_MAP.get(side)))
                .map(side -> RotationUtil.rotateAABB(CONNECTION_AABB, side))
                .forEach(box -> func_185492_a(pos, entityBox, collidingBoxes, box));
    }

    @Override
    public List<ItemStack> getDrops(IBlockAccess world, BlockPos pos, IBlockState state, int fortune) {
        return Collections.emptyList();
    }
}
