/* Copyright (c) 2016 SpaceToad and the BuildCraft team
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/. */
package buildcraft.lib;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import javax.annotation.Nullable;

import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.EntityPlayerSP;
import net.minecraft.client.gui.GuiNewChat;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.Style;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.util.text.event.ClickEvent;
import net.minecraft.world.WorldServer;

import net.minecraftforge.client.event.ModelBakeEvent;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.client.event.TextureStitchEvent;
import net.minecraftforge.common.ForgeVersion;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.event.world.WorldEvent;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.common.ModContainer;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent.ClientTickEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent.Phase;
import net.minecraftforge.fml.common.gameevent.TickEvent.ServerTickEvent;
import net.minecraftforge.fml.common.network.FMLNetworkEvent.ClientConnectedToServerEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import buildcraft.api.registry.EventBuildCraftReload;
import buildcraft.api.tiles.IDebuggable;

import buildcraft.lib.client.guide.GuideManager;
import buildcraft.lib.client.model.ModelHolderRegistry;
import buildcraft.lib.client.reload.ReloadManager;
import buildcraft.lib.client.render.DetachedRenderer;
import buildcraft.lib.client.render.fluid.FluidRenderer;
import buildcraft.lib.client.render.laser.LaserRenderer_BC8;
import buildcraft.lib.client.sprite.SpriteHolderRegistry;
import buildcraft.lib.debug.BCAdvDebugging;
import buildcraft.lib.debug.ClientDebuggables;
import buildcraft.lib.item.ItemDebugger;
import buildcraft.lib.marker.MarkerCache;
import buildcraft.lib.misc.FakePlayerProvider;
import buildcraft.lib.misc.MessageUtil;
import buildcraft.lib.misc.data.ModelVariableData;
import buildcraft.lib.net.MessageDebugRequest;
import buildcraft.lib.net.MessageManager;
import buildcraft.lib.net.cache.BuildCraftObjectCaches;

import buildcraft.core.client.ConfigGuiFactoryBC;

public enum BCLibEventDist {
    INSTANCE;

    @SubscribeEvent
    public static void onEntityJoinWorld(EntityJoinWorldEvent event) {
        Entity entity = event.getEntity();
        if (entity instanceof EntityPlayerMP) {
            EntityPlayerMP playerMP = (EntityPlayerMP) entity;
            // Delay sending join messages to player as it makes it work when in single-player
            MessageUtil.doDelayed(() -> MarkerCache.onPlayerJoinWorld(playerMP));
        }
    }

    @SubscribeEvent
    public static void onWorldUnload(WorldEvent.Unload event) {
        MarkerCache.onWorldUnload(event.getWorld());
        if (event.getWorld() instanceof WorldServer) {
            FakePlayerProvider.INSTANCE.unloadWorld((WorldServer) event.getWorld());
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void onReloadFinish(EventBuildCraftReload.FinishLoad event) {
        // Note: when you need to add server-side listeners the client listeners need to be moved to BCLibProxy
        GuideManager.INSTANCE.onRegistryReload(event);
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void onConnectToServer(ClientConnectedToServerEvent event) {
        BuildCraftObjectCaches.onClientJoinServer();
        // Really obnoxious warning
        if (!BCLib.DEV) {
            /* If people are in a dev environment or have toggled the flag then they probably already know about this */
            Runnable r = () -> {
                try {
                    Thread.sleep(4000);
                } catch (InterruptedException e) {
                    // NO-OP
                }

                String ver;
                if (BCLib.VERSION.startsWith("$")) {
                    ModContainer mod = Loader.instance().getIndexedModList().get(BCLib.MODID);
                    if (mod == null) {
                        ver = "[UNKNOWN-MANUAL-BUILD]";
                    } else {
                        ver = mod.getDisplayVersion();
                        if (ver.startsWith("${")) {
                            // The difference with the above is intentional
                            ver = "[UNKNOWN_MANUAL_BUILD]";
                        }
                    }
                } else {
                    ver = BCLib.VERSION;
                }

                ITextComponent componentVersion = new TextComponentString(ver);
                Style styleVersion = new Style();
                styleVersion.func_150241_a(new ClickEvent(ClickEvent.Action.SUGGEST_COMMAND, BCLib.VERSION));
                // styleVersion.setHoverEvent(new HoverEvent(HoverEvent.Action., valueIn));
                componentVersion.func_150255_a(styleVersion);

                String bodyText = "<!--\n" //
                    + "If your issue is more of a question (like how does a machine work or a sugestion), please use our Discord instead: https://discord.gg/BuildCraft\n"//
                    + "Please fill in all relavant information below.\n"//
                    + "Please do not put the entire log here, upload it on pastebin (https://pastebin.com/) or gist (https://gist.github.com/) and paste here the link.\n"//
                    + "-->\n\n" //
                    + "BuildCraft version: " + BCLib.VERSION + "\n" //
                    + "Forge version: " + ForgeVersion.getVersion() + "\n" //
                    + "Link to crash report or log: {none given}\n" //
                    + "Singleplayer or multiplayer: \n" //
                    + "Steps to reproduce: \n" //
                    + "Additional information: \n"//
                    + "Mod list: \n\n"
                    + Loader.instance().getCrashInformation().replaceAll("UCHIJA+", "Loaded").replace("\t|", "|");

                String githubIssuesUrl;
                try {
                    githubIssuesUrl = "https://github.com/BuildCraft/BuildCraft/issues/new?body="
                        + URLEncoder.encode(bodyText, "UTF-8");
                } catch (UnsupportedEncodingException e) {
                    throw new Error("UTF-8 isn't a valid charset? What?", e);
                }
                ITextComponent componentGithubLink = new TextComponentString("here");
                Style styleGithubLink = new Style();
                styleGithubLink.func_150228_d(Boolean.TRUE);
                styleGithubLink.func_150241_a(new ClickEvent(ClickEvent.Action.OPEN_URL, githubIssuesUrl));
                componentGithubLink.func_150255_a(styleGithubLink);

                TextComponentString textWarn = new TextComponentString("WARNING: BuildCraft ");
                textWarn.func_150257_a(componentVersion);
                textWarn.func_150258_a(" is in ALPHA!");

                TextComponentString textReport = new TextComponentString("  Report BuildCraft bugs you find ");
                textReport.func_150257_a(componentGithubLink);

                TextComponentString textDesc = new TextComponentString("  and include the BuildCraft version ");
                textDesc.func_150257_a(componentVersion);
                textDesc.func_150258_a(" in the description");

                TextComponentString textLag =
                    new TextComponentString("  If you have performance problems then try disabling");
                TextComponentString textConfigLink =
                    new TextComponentString("everything in the BuildCraft perfomance config section.");
                textConfigLink.func_150255_a(new Style() {

                    {
                        func_150228_d(true);
                    }

                    @Override
                    public Style func_150232_l() {
                        return this;
                    }

                    @Override
                    public Style func_150206_m() {
                        return this;
                    }

                    @Override
                    @Nullable
                    public ClickEvent func_150235_h() {
                        // Very hacky, but it technically works
                        StackTraceElement[] trace = new Throwable().getStackTrace();
                        for (StackTraceElement elem : trace) {
                            if (GuiScreen.class.getName().equals(elem.getClassName())) {
                                ConfigGuiFactoryBC.GuiConfigManager newGui =
                                    new ConfigGuiFactoryBC.GuiConfigManager(Minecraft.func_71410_x().field_71462_r);
                                Minecraft.func_71410_x().func_147108_a(newGui);
                                return null;
                            }
                        }
                        return null;
                    }
                });

                ITextComponent[] lines = { textWarn, textReport, textDesc, textLag, textConfigLink };
                GuiNewChat chat = Minecraft.func_71410_x().field_71456_v.func_146158_b();
                for (ITextComponent line : lines) {
                    chat.func_146227_a(line);
                }
            };
            new Thread(r).start();
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void textureStitchPre(TextureStitchEvent.Pre event) {
        ReloadManager.INSTANCE.preReloadResources();
        TextureMap map = event.getMap();
        SpriteHolderRegistry.onTextureStitchPre(map);
        ModelHolderRegistry.onTextureStitchPre(map);
    }

    @SubscribeEvent(priority = EventPriority.LOWEST)
    @SideOnly(Side.CLIENT)
    public static void textureStitchPreLow(TextureStitchEvent.Pre event) {
        FluidRenderer.onTextureStitchPre(event.getMap());
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void textureStitchPost(TextureStitchEvent.Post event) {
        TextureMap map = event.getMap();
        SpriteHolderRegistry.onTextureStitchPost();
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void modelBake(ModelBakeEvent event) {
        SpriteHolderRegistry.exportTextureMap();
        LaserRenderer_BC8.clearModels();
        ModelHolderRegistry.onModelBake();
        ModelVariableData.onModelBake();
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void renderWorldLast(RenderWorldLastEvent event) {
        Minecraft mc = Minecraft.func_71410_x();
        EntityPlayer player = mc.field_71439_g;
        if (player == null) return;
        float partialTicks = event.getPartialTicks();

        DetachedRenderer.INSTANCE.renderWorldLastEvent(player, partialTicks);
    }

    @SubscribeEvent
    public static void serverTick(ServerTickEvent event) {
        if (event.phase == Phase.END) {
            BCAdvDebugging.INSTANCE.onServerPostTick();
            MessageUtil.postTick();
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void clientTick(ClientTickEvent event) {
        if (event.phase == Phase.END) {
            BuildCraftObjectCaches.onClientTick();
            MessageUtil.postTick();
            Minecraft mc = Minecraft.func_71410_x();
            EntityPlayerSP player = mc.field_71439_g;
            if (player != null && ItemDebugger.isShowDebugInfo(player)) {
                RayTraceResult mouseOver = mc.field_71476_x;
                if (mouseOver != null) {
                    IDebuggable debuggable = ClientDebuggables.getDebuggableObject(mouseOver);
                    if (debuggable instanceof TileEntity) {
                        TileEntity tile = (TileEntity) debuggable;
                        MessageManager.sendToServer(new MessageDebugRequest(tile.func_174877_v(), mouseOver.field_178784_b));
                    } else if (debuggable instanceof Entity) {
                        // TODO: Support entities!
                    }
                }
            }
        }
    }
}
