/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.marker;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.storage.WorldSavedData;

import buildcraft.api.core.BCLog;

import buildcraft.lib.misc.NBTUtilBC;

public abstract class MarkerSavedData<S extends MarkerSubCache<C>, C extends MarkerConnection<C>> extends WorldSavedData {
    protected static final boolean DEBUG_FULL = MarkerSubCache.DEBUG_FULL;

    protected final List<BlockPos> markerPositions = new ArrayList<>();
    protected final List<List<BlockPos>> markerConnections = new ArrayList<>();
    private S subCache;

    public MarkerSavedData(String name) {
        super(name);
    }

    @Override
    public void func_76184_a(NBTTagCompound nbt) {
        markerPositions.clear();
        markerConnections.clear();

        NBTTagList positionList = (NBTTagList) nbt.func_74781_a("positions");
        for (int i = 0; i < positionList.func_74745_c(); i++) {
            markerPositions.add(NBTUtilBC.readBlockPos(positionList.func_179238_g(i)));
        }

        NBTTagList connectionList = (NBTTagList) nbt.func_74781_a("connections");
        for (int i = 0; i < connectionList.func_74745_c(); i++) {
            positionList = (NBTTagList) connectionList.func_179238_g(i);
            List<BlockPos> inner = new ArrayList<>();
            markerConnections.add(inner);
            for (int j = 0; j < positionList.func_74745_c(); j++) {
                inner.add(NBTUtilBC.readBlockPos(positionList.func_179238_g(j)));
            }
        }

        if (DEBUG_FULL) {
            BCLog.logger.info("[lib.marker.full] Reading from NBT (" + field_76190_i + ")");
            BCLog.logger.info("[lib.marker.full]  - Positions:");
            for (BlockPos pos : markerPositions) {
                BCLog.logger.info("[lib.marker.full]   - " + pos);
            }
            BCLog.logger.info("[lib.marker.full]  - Connections:");
            for (List<BlockPos> list : markerConnections) {
                BCLog.logger.info("[lib.marker.full]   - Single Connection:");
                for (BlockPos pos : list) {
                    BCLog.logger.info("[lib.marker.full]     - " + pos);
                }
            }
        }
    }

    @Override
    public NBTTagCompound func_189551_b(NBTTagCompound nbt) {
        markerPositions.clear();
        markerConnections.clear();

        markerPositions.addAll(subCache.getAllMarkers());
        for (C connection : subCache.getConnections()) {
            markerConnections.add(new ArrayList<>(connection.getMarkerPositions()));
        }

        NBTTagList positionList = new NBTTagList();
        for (BlockPos p : markerPositions) {
            positionList.func_74742_a(NBTUtilBC.writeBlockPos(p));
        }
        nbt.func_74782_a("positions", positionList);

        NBTTagList connectionList = new NBTTagList();
        for (List<BlockPos> connection : markerConnections) {
            NBTTagList inner = new NBTTagList();
            for (BlockPos p : connection) {
                inner.func_74742_a(NBTUtilBC.writeBlockPos(p));
            }
            connectionList.func_74742_a(inner);
        }
        nbt.func_74782_a("connections", connectionList);

        if (DEBUG_FULL) {
            BCLog.logger.info("[lib.marker.full] Writing to NBT (" + field_76190_i + ")");
            BCLog.logger.info("[lib.marker.full]  - Positions:");
            for (BlockPos pos : markerPositions) {
                BCLog.logger.info("[lib.marker.full]   - " + pos);
            }
            BCLog.logger.info("[lib.marker.full]  - Connections:");
            for (List<BlockPos> list : markerConnections) {
                BCLog.logger.info("[lib.marker.full]   - Single Connection:");
                for (BlockPos pos : list) {
                    BCLog.logger.info("[lib.marker.full]     - " + pos);
                }
            }
        }

        return nbt;
    }

    @Override
    public boolean func_76188_b() {
        return true;
    }

    public final void setCache(S subCache) {
        this.subCache = subCache;
    }
}
